function parfor_1_read_individual_fcs_get_downsampling_info(filename,arcsinh_cofactor,downsampling_scaling_factor,used_markers,is_normalize, normalize_weight_factor)

    [fcsdat, fcshdr, fcsdatscaled] = fca_readfcs(filename);
    data = flow_arcsinh(fcsdat',arcsinh_cofactor);
    data = data(:,1:min(end,500000));
    
    display(['Read and downsample fcs file: '])
    display(['   ',filename,]);
    display(['   ',num2str(size(data,2)), ' cells']);
    for i=1:length(fcshdr.par), 
        marker_names{i,1} = fcshdr.par(i).name2;  
        if isequal(unique(fcshdr.par(i).name2),' ')
            marker_names{i,1} = fcshdr.par(i).name;  
        end
    end
    if exist('is_normalize') && is_normalize==1
        if ~exist('normalize_weight_factor')
            data = flow_normalize(data);
        else
            data = flow_normalize(data, normalize_weight_factor);
        end
    end
    if iscell(used_markers)
        [C,IA,IB] = intersect(used_markers,marker_names);
        if length(used_markers)~=length(C)
            error('Some input markers do not exist in at least one FCS file');
        end
        used_markers = sort(IB);
    end
    [is_keep,local_density,keep_prob,kernel_width] = parfor_downsample_flow(data(used_markers,:),5,downsampling_scaling_factor);
    save([filename(1:end-4),'.mat'],'data','marker_names','local_density','used_markers','kernel_width')
return



function [local_density,kernel_width] = downsample_flow_c(data,kernel_width_para)

Nchannels = size(data,1);
Ncells = size(data,2);

min_dist = zeros(1,min(size(data,2),2000));

fprintf('   finding empirical dist of the min distance between cells ... %4d',0);
for i=1:length(min_dist)
    ind = randsample(1:size(data,2),1);
    all_dist = sum(abs(repmat(data(:,ind),1,size(data,2)-1) - data(:,setdiff(1:end,ind))),1);
    min_dist(i) = min(all_dist);
    fprintf('\b\b\b\b%4d',i);
end
fprintf('\n');

med_min_dist = median(min_dist);
kernel_width = kernel_width_para * med_min_dist;
fprintf('   For this %d channel data, KERNEL_WIDTH is %3.3f\n', size(data,1),kernel_width);

% if size(data,2)<=2000
%     ind = 1:size(data,2);
% else
%     ind = randsample(1:size(data,2),2000);
% end
% min_dist = zeros(1,length(ind));
% fprintf('   finding empirical dist of the min distance between cells ... %4d',0);
% for i=1:length(ind)
%     min_dist(i) = compute_min_dist_for_step1(data(:,setdiff(1:end,ind(i))), Ncells-1, data(:,ind(i)), Nchannels);
%     fprintf('\b\b\b\b%4d',i);
% end 
% fprintf('\n');
% med_min_dist = median(min_dist);
% kernel_width = med_min_dist*kernel_width_para;

cheating_neighborhood = med_min_dist;
[local_density] = compute_local_density(data, Nchannels, Ncells, kernel_width, cheating_neighborhood);
return




function [is_keep,local_density,keep_prob,kernel_width] = parfor_downsample_flow(data,target_prctile,kernel_width_para)


if ~exist('kernel_width_para')
    kernel_width_para = 10;
end
if ~exist('target_prctile')
    target_prctile = 5;
end

min_dist = zeros(1,min(size(data,2),2000));

fprintf('   finding empirical dist of the min distance between cells ... %4d',0);
for i=1:length(min_dist)
    ind = randsample(1:size(data,2),1);
    all_dist = sum(abs(repmat(data(:,ind),1,size(data,2)-1) - data(:,setdiff(1:end,ind))),1);
    min_dist(i) = min(all_dist);
    fprintf('\b\b\b\b%4d',i);
end
fprintf('\n');

med_min_dist = median(min_dist);
kernel_width = kernel_width_para * med_min_dist;
fprintf('   For this %d channel data, KERNEL_WIDTH is %3.3f\n', size(data,1),kernel_width);

local_density = zeros(1,size(data,2));
fprintf('   finding local density for each cell ... %4d%%',0);
for i=1:size(data,2)
    if local_density(i)==0
        all_dist = sum(abs(repmat(data(:,i),1,size(data,2)) - data),1);
        local_density(i) = sum(max(kernel_width-all_dist,0)/kernel_width); % sum(kernel_width>=all_dist); 
        local_density(all_dist< med_min_dist) = local_density(i);
    end
    if mod(i,1000)==1 || i == size(data,2)
        fprintf('\b\b\b\b\b%4d%%',floor(i/length(local_density)*100));
    end
end
fprintf('\n', size(data,1),kernel_width);

target_density = prctile(local_density,target_prctile);
keep_prob = (target_density./local_density);
fprintf('   Down-sample cells ... \n');
is_keep = rand(1,size(data,2))<keep_prob;

return



